@@ -1,3 +1,6 @@
+#ifndef QT_FEATURES_Widgets_src_widgets_qtwidgets_config_p_h_H
+#define QT_FEATURES_Widgets_src_widgets_qtwidgets_config_p_h_H
+
 #define QT_FEATURE_gtk3 1
 
 #define QT_FEATURE_style_fusion 1
@@ -16,3 +19,5 @@
 
 #define QT_FEATURE_widgettextcontrol 1
 
+
+#endif // QT_FEATURES_Widgets_src_widgets_qtwidgets_config_p_h_H
